@@ -175,10 +175,10 @@ wald.fct 		<- function(
   if(estimation == "onestep"){
 #    w.stat		<- n*crossprod(coef.hat, tcrossprod(MASS::ginv(vcov.hat), t(coef.hat)) ) *
 #				(as.vector(crossprod(do.call(res.1s_temp, what = "c"), do.call(Szero.j, what = "c"), na.rm = TRUE) /(n*Time - sum(n.inst)+7)))						#[M:] Stata results with different dof-correction
-    w.stat		<- n * Matrix::crossprod(coef.hat, Matrix::tcrossprod(solve(vcov.hat), Matrix::t(coef.hat)) ) *
+    w.stat		<- n * Matrix::crossprod(coef.hat, Matrix::tcrossprod(MASS::ginv(as.matrix(vcov.hat)), Matrix::t(coef.hat)) ) *
 				(as.vector(Matrix::crossprod(do.call(Szero.j, what = "c"), do.call(Szero.j, what = "c"), na.rm = TRUE)) /(sum(!is.na(dat.na[, varname.y])) - sum(n.inst)))		#[M:] Adjusted dof-correction (for missing observations)
   } else{
-    w.stat		<- crossprod(coef.hat, tcrossprod(solve(vcov.hat), t(coef.hat)) )
+    w.stat		<- crossprod(coef.hat, tcrossprod(MASS::ginv(as.matrix(vcov.hat)), t(coef.hat)) )
   }
 
   names(w.stat)	<- "chisq"
